/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.appkit.NSCursor;

import dstep.appkit.NSColor;
import dstep.appkit.NSEvent;
import dstep.appkit.NSImage;
import dstep.foundation.NSCoder;
import dstep.foundation.NSGeometry;
import dstep.foundation.NSObject;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

class NSCursor : NSObject, INSCoding
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}
	
	this (NSCoder aDecoder)
	{
		super(typeof(this).alloc.initWithCoder(aDecoder).objcObject);
	}
	
	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}
	
	typeof(this) initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(typeof(this), "initWithCoder:", NSCoder)(aDecoder);
	}

	static NSCursor currentCursor ()
	{
		return invokeObjcSelfClass!(NSCursor, "currentCursor");
	}

	static NSCursor arrowCursor ()
	{
		return invokeObjcSelfClass!(NSCursor, "arrowCursor");
	}

	static NSCursor IBeamCursor ()
	{
		return invokeObjcSelfClass!(NSCursor, "IBeamCursor");
	}

	static NSCursor pointingHandCursor ()
	{
		return invokeObjcSelfClass!(NSCursor, "pointingHandCursor");
	}

	static NSCursor closedHandCursor ()
	{
		return invokeObjcSelfClass!(NSCursor, "closedHandCursor");
	}

	static NSCursor openHandCursor ()
	{
		return invokeObjcSelfClass!(NSCursor, "openHandCursor");
	}

	static NSCursor resizeLeftCursor ()
	{
		return invokeObjcSelfClass!(NSCursor, "resizeLeftCursor");
	}

	static NSCursor resizeRightCursor ()
	{
		return invokeObjcSelfClass!(NSCursor, "resizeRightCursor");
	}

	static NSCursor resizeLeftRightCursor ()
	{
		return invokeObjcSelfClass!(NSCursor, "resizeLeftRightCursor");
	}

	static NSCursor resizeUpCursor ()
	{
		return invokeObjcSelfClass!(NSCursor, "resizeUpCursor");
	}

	static NSCursor resizeDownCursor ()
	{
		return invokeObjcSelfClass!(NSCursor, "resizeDownCursor");
	}

	static NSCursor resizeUpDownCursor ()
	{
		return invokeObjcSelfClass!(NSCursor, "resizeUpDownCursor");
	}

	static NSCursor crosshairCursor ()
	{
		return invokeObjcSelfClass!(NSCursor, "crosshairCursor");
	}

	static NSCursor disappearingItemCursor ()
	{
		return invokeObjcSelfClass!(NSCursor, "disappearingItemCursor");
	}

	NSCursor initWithImage (NSImage newImage, NSPoint aPoint)
	{
		id result = invokeObjcSelf!(id, "initWithImage:hotSpot:", NSImage, NSPoint)(newImage, aPoint);
		return result is this.objcObject ? this : (result !is null ? new NSCursor(result) : null);
	}

	this (NSImage newImage, NSPoint aPoint)
	{
		super(NSCursor.alloc.initWithImage(newImage, aPoint).objcObject);
	}

	NSCursor initWithImage (NSImage newImage, NSColor fg, NSColor bg, NSPoint hotSpot)
	{
		id result = invokeObjcSelf!(id, "initWithImage:foregroundColorHint:backgroundColorHint:hotSpot:", NSImage, NSColor, NSColor, NSPoint)(newImage, fg, bg, hotSpot);
		return result is this.objcObject ? this : (result !is null ? new NSCursor(result) : null);
	}

	this (NSImage newImage, NSColor fg, NSColor bg, NSPoint hotSpot)
	{
		super(NSCursor.alloc.initWithImage(newImage, fg, bg, hotSpot).objcObject);
	}

	static void hide ()
	{
		return invokeObjcSelfClass!(void, "hide");
	}

	static void unhide ()
	{
		return invokeObjcSelfClass!(void, "unhide");
	}

	static void setHiddenUntilMouseMoves (bool flag)
	{
		return invokeObjcSelfClass!(void, "setHiddenUntilMouseMoves:", bool)(flag);
	}

	static void pop_static ()
	{
		return invokeObjcSelfClass!(void, "pop");
	}

	NSImage image ()
	{
		return invokeObjcSelf!(NSImage, "image");
	}

	NSPoint hotSpot ()
	{
		return invokeObjcSelf!(NSPoint, "hotSpot");
	}

	void push ()
	{
		return invokeObjcSelf!(void, "push");
	}

	void pop ()
	{
		return invokeObjcSelf!(void, "pop");
	}

	void set ()
	{
		return invokeObjcSelf!(void, "set");
	}

	void setOnMouseExited (bool flag)
	{
		return invokeObjcSelf!(void, "setOnMouseExited:", bool)(flag);
	}

	void setOnMouseEntered (bool flag)
	{
		return invokeObjcSelf!(void, "setOnMouseEntered:", bool)(flag);
	}

	bool isSetOnMouseExited ()
	{
		return invokeObjcSelf!(bool, "isSetOnMouseExited");
	}

	bool isSetOnMouseEntered ()
	{
		return invokeObjcSelf!(bool, "isSetOnMouseEntered");
	}

	void mouseEntered (NSEvent theEvent)
	{
		return invokeObjcSelf!(void, "mouseEntered:", NSEvent)(theEvent);
	}

	void mouseExited (NSEvent theEvent)
	{
		return invokeObjcSelf!(void, "mouseExited:", NSEvent)(theEvent);
	}

}

