/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.appkit.NSSpellChecker;

import dstep.foundation.NSArray;
import dstep.appkit.NSPanel;
import dstep.appkit.NSView;
import dstep.foundation.NSDictionary;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSObject;
import dstep.foundation.NSRange;
import dstep.foundation.NSString;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

class NSSpellChecker : NSObject
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}

	static NSSpellChecker sharedSpellChecker ()
	{
		return invokeObjcSelfClass!(NSSpellChecker, "sharedSpellChecker");
	}

	static bool sharedSpellCheckerExists ()
	{
		return invokeObjcSelfClass!(bool, "sharedSpellCheckerExists");
	}

	static NSInteger uniqueSpellDocumentTag ()
	{
		return invokeObjcSelfClass!(NSInteger, "uniqueSpellDocumentTag");
	}

	NSRange checkSpellingOfString (NSString stringToCheck, NSInteger startingOffset, NSString language, bool wrapFlag, NSInteger tag, NSInteger* wordCount)
	{
		return invokeObjcSelf!(NSRange, "checkSpellingOfString:startingAt:language:wrap:inSpellDocumentWithTag:wordCount:", NSString, NSInteger, NSString, bool, NSInteger, NSInteger*)(stringToCheck, startingOffset, language, wrapFlag, tag, wordCount);
	}

	NSRange checkSpellingOfString (NSString stringToCheck, NSInteger startingOffset)
	{
		return invokeObjcSelf!(NSRange, "checkSpellingOfString:startingAt:", NSString, NSInteger)(stringToCheck, startingOffset);
	}

	NSInteger countWordsInString (NSString stringToCount, NSString language)
	{
		return invokeObjcSelf!(NSInteger, "countWordsInString:language:", NSString, NSString)(stringToCount, language);
	}

	NSRange checkGrammarOfString (NSString stringToCheck, NSInteger startingOffset, NSString language, bool wrapFlag, NSInteger tag, NSArray** details)
	{
		return invokeObjcSelf!(NSRange, "checkGrammarOfString:startingAt:language:wrap:inSpellDocumentWithTag:details:", NSString, NSInteger, NSString, bool, NSInteger, NSArray**)(stringToCheck, startingOffset, language, wrapFlag, tag, details);
	}

	void updateSpellingPanelWithMisspelledWord (NSString word)
	{
		return invokeObjcSelf!(void, "updateSpellingPanelWithMisspelledWord:", NSString)(word);
	}

	void updateSpellingPanelWithGrammarString (NSString string, NSDictionary detail)
	{
		return invokeObjcSelf!(void, "updateSpellingPanelWithGrammarString:detail:", NSString, NSDictionary)(string, detail);
	}

	NSPanel spellingPanel ()
	{
		return invokeObjcSelf!(NSPanel, "spellingPanel");
	}

	NSView accessoryView ()
	{
		return invokeObjcSelf!(NSView, "accessoryView");
	}

	void setAccessoryView (NSView aView)
	{
		return invokeObjcSelf!(void, "setAccessoryView:", NSView)(aView);
	}

	void ignoreWord (NSString wordToIgnore, NSInteger tag)
	{
		return invokeObjcSelf!(void, "ignoreWord:inSpellDocumentWithTag:", NSString, NSInteger)(wordToIgnore, tag);
	}

	NSArray ignoredWordsInSpellDocumentWithTag (NSInteger tag)
	{
		return invokeObjcSelf!(NSArray, "ignoredWordsInSpellDocumentWithTag:", NSInteger)(tag);
	}

	void setIgnoredWords (NSArray words, NSInteger tag)
	{
		return invokeObjcSelf!(void, "setIgnoredWords:inSpellDocumentWithTag:", NSArray, NSInteger)(words, tag);
	}

	NSArray guessesForWord (NSString word)
	{
		return invokeObjcSelf!(NSArray, "guessesForWord:", NSString)(word);
	}

	NSArray completionsForPartialWordRange (NSRange range, NSString string, NSString language, NSInteger tag)
	{
		return invokeObjcSelf!(NSArray, "completionsForPartialWordRange:inString:language:inSpellDocumentWithTag:", NSRange, NSString, NSString, NSInteger)(range, string, language, tag);
	}

	void closeSpellDocumentWithTag (NSInteger tag)
	{
		return invokeObjcSelf!(void, "closeSpellDocumentWithTag:", NSInteger)(tag);
	}

	NSString language ()
	{
		return invokeObjcSelf!(NSString, "language");
	}

	bool setLanguage (NSString language)
	{
		return invokeObjcSelf!(bool, "setLanguage:", NSString)(language);
	}

	NSArray availableLanguages ()
	{
		return invokeObjcSelf!(NSArray, "availableLanguages");
	}

	void setWordFieldStringValue (NSString aString)
	{
		return invokeObjcSelf!(void, "setWordFieldStringValue:", NSString)(aString);
	}

	void learnWord (NSString word)
	{
		return invokeObjcSelf!(void, "learnWord:", NSString)(word);
	}

	bool hasLearnedWord (NSString word)
	{
		return invokeObjcSelf!(bool, "hasLearnedWord:", NSString)(word);
	}

	void unlearnWord (NSString word)
	{
		return invokeObjcSelf!(void, "unlearnWord:", NSString)(word);
	}
}

