/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.appkit.NSTabView;

import dstep.foundation.NSArray;
import dstep.appkit.NSCell;
import dstep.appkit.NSFont;
import dstep.appkit.NSTabViewItem;
import dstep.appkit.NSView;
import dstep.foundation.NSGeometry;
import dstep.foundation.NSObjCRuntime;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

typedef NSUInteger NSTabViewType;

enum : NSUInteger
{
	NSTopTabsBezelBorder = 0,
	NSLeftTabsBezelBorder = 1,
	NSBottomTabsBezelBorder = 2,
	NSRightTabsBezelBorder = 3,
	NSNoTabsBezelBorder = 4,
	NSNoTabsLineBorder = 5,
	NSNoTabsNoBorder = 6
}

class NSTabView : NSView
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}

	void selectTabViewItem (NSTabViewItem tabViewItem)
	{
		return invokeObjcSelf!(void, "selectTabViewItem:", NSTabViewItem)(tabViewItem);
	}

	void selectTabViewItemAtIndex (NSInteger index)
	{
		return invokeObjcSelf!(void, "selectTabViewItemAtIndex:", NSInteger)(index);
	}

	void selectTabViewItemWithIdentifier (Object identifier)
	{
		return invokeObjcSelf!(void, "selectTabViewItemWithIdentifier:", Object)(identifier);
	}

	void takeSelectedTabViewItemFromSender (Object sender)
	{
		return invokeObjcSelf!(void, "takeSelectedTabViewItemFromSender:", Object)(sender);
	}

	void selectFirstTabViewItem (Object sender)
	{
		return invokeObjcSelf!(void, "selectFirstTabViewItem:", Object)(sender);
	}

	void selectLastTabViewItem (Object sender)
	{
		return invokeObjcSelf!(void, "selectLastTabViewItem:", Object)(sender);
	}

	void selectNextTabViewItem (Object sender)
	{
		return invokeObjcSelf!(void, "selectNextTabViewItem:", Object)(sender);
	}

	void selectPreviousTabViewItem (Object sender)
	{
		return invokeObjcSelf!(void, "selectPreviousTabViewItem:", Object)(sender);
	}

	NSTabViewItem selectedTabViewItem ()
	{
		return invokeObjcSelf!(NSTabViewItem, "selectedTabViewItem");
	}

	NSFont font ()
	{
		return invokeObjcSelf!(NSFont, "font");
	}

	uint tabViewType ()
	{
		return invokeObjcSelf!(uint, "tabViewType");
	}

	NSArray tabViewItems ()
	{
		return invokeObjcSelf!(NSArray, "tabViewItems");
	}

	bool allowsTruncatedLabels ()
	{
		return invokeObjcSelf!(bool, "allowsTruncatedLabels");
	}

	NSSize minimumSize ()
	{
		return invokeObjcSelf!(NSSize, "minimumSize");
	}

	bool drawsBackground ()
	{
		return invokeObjcSelf!(bool, "drawsBackground");
	}

	uint controlTint ()
	{
		return invokeObjcSelf!(uint, "controlTint");
	}

	uint controlSize ()
	{
		return invokeObjcSelf!(uint, "controlSize");
	}

	void setFont (NSFont font)
	{
		return invokeObjcSelf!(void, "setFont:", NSFont)(font);
	}

	void setTabViewType (uint tabViewType)
	{
		return invokeObjcSelf!(void, "setTabViewType:", uint)(tabViewType);
	}

	void setAllowsTruncatedLabels (bool allowTruncatedLabels)
	{
		return invokeObjcSelf!(void, "setAllowsTruncatedLabels:", bool)(allowTruncatedLabels);
	}

	void setDrawsBackground (bool flag)
	{
		return invokeObjcSelf!(void, "setDrawsBackground:", bool)(flag);
	}

	void setControlTint (uint controlTint)
	{
		return invokeObjcSelf!(void, "setControlTint:", uint)(controlTint);
	}

	void setControlSize (uint controlSize)
	{
		return invokeObjcSelf!(void, "setControlSize:", uint)(controlSize);
	}

	void addTabViewItem (NSTabViewItem tabViewItem)
	{
		return invokeObjcSelf!(void, "addTabViewItem:", NSTabViewItem)(tabViewItem);
	}

	void insertTabViewItem (NSTabViewItem tabViewItem, NSInteger index)
	{
		return invokeObjcSelf!(void, "insertTabViewItem:atIndex:", NSTabViewItem, NSInteger)(tabViewItem, index);
	}

	void removeTabViewItem (NSTabViewItem tabViewItem)
	{
		return invokeObjcSelf!(void, "removeTabViewItem:", NSTabViewItem)(tabViewItem);
	}

	void setDelegate (Object anObject)
	{
		return invokeObjcSelf!(void, "setDelegate:", Object)(anObject);
	}

	Object delegate_ ()
	{
		return invokeObjcSelf!(Object, "delegate");
	}

	NSTabViewItem tabViewItemAtPoint (NSPoint point)
	{
		return invokeObjcSelf!(NSTabViewItem, "tabViewItemAtPoint:", NSPoint)(point);
	}

	NSRect contentRect ()
	{
		return invokeObjcSelf!(NSRect, "contentRect");
	}

	NSInteger numberOfTabViewItems ()
	{
		return invokeObjcSelf!(NSInteger, "numberOfTabViewItems");
	}

	NSInteger indexOfTabViewItem (NSTabViewItem tabViewItem)
	{
		return invokeObjcSelf!(NSInteger, "indexOfTabViewItem:", NSTabViewItem)(tabViewItem);
	}

	NSTabViewItem tabViewItemAtIndex (NSInteger index)
	{
		return invokeObjcSelf!(NSTabViewItem, "tabViewItemAtIndex:", NSInteger)(index);
	}

	NSInteger indexOfTabViewItemWithIdentifier (Object identifier)
	{
		return invokeObjcSelf!(NSInteger, "indexOfTabViewItemWithIdentifier:", Object)(identifier);
	}
}

const TNSTabViewDelegate = `

	bool tabView (NSTabView tabView, NSTabViewItem tabViewItem)
	{
		return invokeObjcSelf!(bool, "tabView:shouldSelectTabViewItem:", NSTabView, NSTabViewItem)(tabView, tabViewItem);
	}

	void tabView (NSTabView tabView, NSTabViewItem tabViewItem)
	{
		return invokeObjcSelf!(void, "tabView:willSelectTabViewItem:", NSTabView, NSTabViewItem)(tabView, tabViewItem);
	}

	void tabView (NSTabView tabView, NSTabViewItem tabViewItem)
	{
		return invokeObjcSelf!(void, "tabView:didSelectTabViewItem:", NSTabView, NSTabViewItem)(tabView, tabViewItem);
	}

	void tabViewDidChangeNumberOfTabViewItems (NSTabView TabView)
	{
		return invokeObjcSelf!(void, "tabViewDidChangeNumberOfTabViewItems:", NSTabView)(TabView);
	}

	//mixin ObjcBindMethod!(tabView, "tabView:shouldSelectTabViewItem:");
	//mixin ObjcBindMethod!(tabView, "tabView:willSelectTabViewItem:");
	//mixin ObjcBindMethod!(tabView, "tabView:didSelectTabViewItem:");
	//mixin ObjcBindMethod!(tabViewDidChangeNumberOfTabViewItems, "tabViewDidChangeNumberOfTabViewItems:");

`;

