/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSHTTPCookieStorage;

import dstep.foundation.NSArray;
import dstep.foundation.NSHTTPCookie;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSObject;
import dstep.foundation.NSString;
import dstep.foundation.NSURL;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

import bindings = dstep.foundation.NSHTTPCookieStorage_bindings;

alias NSUInteger NSHTTPCookieAcceptPolicy;

private
{
	NSString NSHTTPCookieManagerAcceptPolicyChangedNotification_;
	NSString NSHTTPCookieManagerCookiesChangedNotification_;
}

enum
{
	NSHTTPCookieAcceptPolicyAlways,
	NSHTTPCookieAcceptPolicyNever,
	NSHTTPCookieAcceptPolicyOnlyFromMainDocumentDomain
}

NSString NSHTTPCookieManagerAcceptPolicyChangedNotification ()
{
	if (NSHTTPCookieManagerAcceptPolicyChangedNotification_)
		return NSHTTPCookieManagerAcceptPolicyChangedNotification_;
		
	return NSHTTPCookieManagerAcceptPolicyChangedNotification_ = new NSString(bindings.NSHTTPCookieManagerAcceptPolicyChangedNotification);
}

NSString NSHTTPCookieManagerCookiesChangedNotification ()
{
	if (NSHTTPCookieManagerCookiesChangedNotification_)
		return NSHTTPCookieManagerCookiesChangedNotification_;
		
	return NSHTTPCookieManagerCookiesChangedNotification_ = new NSString(bindings.NSHTTPCookieManagerCookiesChangedNotification);
}

class NSHTTPCookieStorage : NSObject
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	static NSHTTPCookieStorage sharedHTTPCookieStorage ()
	{
		return invokeObjcSuperClass!(NSHTTPCookieStorage, "sharedHTTPCookieStorage");
	}

	NSArray cookies ()
	{
		return invokeObjcSelf!(NSArray, "cookies");
	}

	void setCookie (NSHTTPCookie cookie)
	{
		return invokeObjcSelf!(void, "setCookie:", NSHTTPCookie)(cookie);
	}

	void deleteCookie (NSHTTPCookie cookie)
	{
		return invokeObjcSelf!(void, "deleteCookie:", NSHTTPCookie)(cookie);
	}

	NSArray cookiesForURL (NSURL URL)
	{
		return invokeObjcSelf!(NSArray, "cookiesForURL:", NSURL)(URL);
	}

	void setCookies (NSArray cookies, NSURL URL, NSURL mainDocumentURL)
	{
		return invokeObjcSelf!(void, "setCookies:forURL:mainDocumentURL:", NSArray, NSURL, NSURL)(cookies, URL, mainDocumentURL);
	}

	uint cookieAcceptPolicy ()
	{
		return invokeObjcSelf!(uint, "cookieAcceptPolicy");
	}

	void setCookieAcceptPolicy (uint cookieAcceptPolicy)
	{
		return invokeObjcSelf!(void, "setCookieAcceptPolicy:", uint)(cookieAcceptPolicy);
	}
}

